// -*- mode:java; encoding:utf-8 -*-
// vim:set fileencoding=utf-8:
// https://ateraimemo.com/Swing/ActionEventModifiers.html

package example;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.lang.invoke.MethodHandles;
import java.util.logging.LogRecord;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;
import java.util.logging.StreamHandler;
import javax.swing.*;

public final class MainPanel extends JPanel {
  public static final String LOGGER_NAME = MethodHandles.lookup().lookupClass().getName();
  private static final Logger LOGGER = Logger.getLogger(LOGGER_NAME);

  private MainPanel() {
    super(new BorderLayout());
    LOGGER.setUseParentHandlers(false);
    JTextArea textArea = new JTextArea();
    LOGGER.addHandler(new TextAreaHandler(new TextAreaOutputStream(textArea)));

    JTextField field = new JTextField(20);
    KeyStroke ks = KeyStroke.getKeyStroke(KeyEvent.VK_B, InputEvent.SHIFT_DOWN_MASK);
    String key = "beep";
    field.getInputMap().put(ks, key);
    field.getActionMap().put(key, new AbstractAction() {
      @Override public void actionPerformed(ActionEvent e) {
        Toolkit.getDefaultToolkit().beep();
        // UIManager.getLookAndFeel().provideErrorFeedback((Component) e.getSource());
      }
    });
    field.addKeyListener(new KeyAdapter() {
      @Override public void keyPressed(KeyEvent e) {
        // InputEvent.SHIFT_MASK @Deprecated(since="9")
        // boolean shiftActive = (e.getModifiers() & InputEvent.SHIFT_MASK) != 0;
        boolean shiftActive = (e.getModifiersEx() & InputEvent.SHIFT_DOWN_MASK) != 0;
        if (e.getKeyCode() == KeyEvent.VK_N && shiftActive) {
          // or: if (e.getKeyCode() == KeyEvent.VK_N && e.isShiftDown()) {
          UIManager.getLookAndFeel().provideErrorFeedback(e.getComponent());
        }
      }
    });

    JButton button = new JButton("TEST: ActionEvent#getModifiers()");
    button.addActionListener(e -> {
      // BAD EXAMPLE: boolean isShiftDown = (e.getModifiers() & InputEvent.SHIFT_MASK) != 0;
      // Always use ActionEvent.*_MASK instead of InputEvent.*_MASK in ActionListener
      boolean isShiftDown = (e.getModifiers() & ActionEvent.SHIFT_MASK) != 0;
      if (isShiftDown) {
        LOGGER.info(() -> "JButton: Shift is Down");
      } else {
        LOGGER.info(() -> "JButton: Shift is Up");
      }
      if ((e.getModifiers() & AWTEvent.MOUSE_EVENT_MASK) != 0) {
        LOGGER.info(() -> "JButton: Mouse event mask");
      }
    });

    JMenuBar menuBar = new JMenuBar();
    JMenu menu = menuBar.add(new JMenu("Test"));
    menu.setMnemonic(KeyEvent.VK_T);
    JMenuItem item = menu.add(new AbstractAction("beep") {
      @Override public void actionPerformed(ActionEvent e) {
        Toolkit.getDefaultToolkit().beep();
      }
    });
    item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_1, InputEvent.SHIFT_DOWN_MASK));
    item.setMnemonic(KeyEvent.VK_I);
    item.addActionListener(e -> {
      boolean isShiftDown = (e.getModifiers() & ActionEvent.SHIFT_MASK) != 0;
      if (isShiftDown) {
        LOGGER.info(() -> "JMenuItem: Shift is Down");
      } else {
        LOGGER.info(() -> "JMenuItem: Shift is Up");
      }
      if ((e.getModifiers() & AWTEvent.MOUSE_EVENT_MASK) != 0) {
        LOGGER.info(() -> "JMenuItem: Mouse event mask");
      }
    });

    EventQueue.invokeLater(() -> {
      JRootPane root = getRootPane();
      root.setJMenuBar(menuBar);
      root.setDefaultButton(button);
    });

    JPanel p = new JPanel(new GridLayout(2, 1, 5, 5));
    p.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
    p.add(field);
    p.add(button);

    add(p, BorderLayout.NORTH);
    add(new JScrollPane(textArea));
    setPreferredSize(new Dimension(320, 240));
  }

  public static void main(String[] args) {
    EventQueue.invokeLater(MainPanel::createAndShowGui);
  }

  private static void createAndShowGui() {
    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    } catch (UnsupportedLookAndFeelException ignored) {
      Toolkit.getDefaultToolkit().beep();
    } catch (ClassNotFoundException | InstantiationException | IllegalAccessException ex) {
      ex.printStackTrace();
      return;
    }
    JFrame frame = new JFrame("ActionEventModifiers");
    frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
    frame.getContentPane().add(new MainPanel());
    frame.pack();
    frame.setLocationRelativeTo(null);
    frame.setVisible(true);
  }
}

class TextAreaOutputStream extends OutputStream {
  private final ByteArrayOutputStream buffer = new ByteArrayOutputStream();
  private final JTextArea textArea;

  protected TextAreaOutputStream(JTextArea textArea) {
    super();
    this.textArea = textArea;
  }

  // // Java 10:
  // @Override public void flush() {
  //   textArea.append(buffer.toString(StandardCharsets.UTF_8));
  //   buffer.reset();
  // }

  @Override public void flush() throws IOException {
    textArea.append(buffer.toString("UTF-8"));
    buffer.reset();
  }

  @Override public void write(int b) {
    buffer.write(b);
  }

  @Override public void write(byte[] b, int off, int len) {
    buffer.write(b, off, len);
  }
}

class TextAreaHandler extends StreamHandler {
  private void configure() {
    setFormatter(new SimpleFormatter());
    try {
      setEncoding("UTF-8");
    } catch (IOException ex) {
      try {
        setEncoding(null);
      } catch (IOException ex2) {
        // doing a setEncoding with null should always work.
        assert false;
      }
    }
  }

  protected TextAreaHandler(OutputStream os) {
    super();
    configure();
    setOutputStream(os);
  }

  // [UnsynchronizedOverridesSynchronized]
  // Unsynchronized method publish overrides synchronized method in StreamHandler
  @SuppressWarnings("PMD.AvoidSynchronizedAtMethodLevel")
  @Override public synchronized void publish(LogRecord logRecord) {
    super.publish(logRecord);
    flush();
  }

  // [UnsynchronizedOverridesSynchronized]
  // Unsynchronized method close overrides synchronized method in StreamHandler
  @SuppressWarnings("PMD.AvoidSynchronizedAtMethodLevel")
  @Override public synchronized void close() {
    flush();
  }
}
